@@ -70,7 +70,7 @@ class ArchiveNode():
     @staticmethod
     def _get_relation_fields(node, fields=None):
         """
-        Build a list relationship fields by ascending up the node tree.
+        Build a list of relationship fields by ascending up the node tree.
 
         The resulting list can then be used to construct a Django
         "query param" filter field.
@@ -85,24 +85,30 @@ class ArchiveNode():
 
         return fields
 
-    def expand(self, models=None):
+    def expand(self, parsed_rel_fields=None):
         """
         Expand relationships from the node.
-
         Uses introspection to build a single-direction tree of related models.
         """
 
-        # Instantiate a model list. Prevents circular dependences in the tree.
-        if models is None:
-            models = [self.model]
+        # Instantiate a parsed_rel_fields list.
+        if parsed_rel_fields is None:
+            parsed_rel_fields = []
+
+        def _get_f_key(model, f):
+            rel_model_name = f.related_model.__name__.lower() \
+                if f.related_model else None
+            return "{}:{}".format(model.__name__.lower(), rel_model_name)
 
         # Get the current node's (model's) dependent relationship fields.
         # 1. The field must be a relationship field.
         # 2. The related field's model must be a BaseModel type.
+        # 3. The relationship must not have been parsed already.
         fields = [f for f in self.model._meta.get_fields()
-                 if (f.related_model not in models
-                 and (f.one_to_many or f.one_to_one)
-                 and (issubclass(f.related_model, ArchiveModel)))]
+                 if (f.related_model != self.model
+                     and (_get_f_key(self.model, f) not in parsed_rel_fields)
+                     and (f.one_to_many or f.one_to_one)
+                     and (issubclass(f.related_model, ArchiveModel)))]
 
         for f in fields:
             if hasattr(f, 'field'):
@@ -112,18 +118,41 @@ class ArchiveNode():
 
             # Create a new node with the correct parent and relationship field.
             node = ArchiveNode(
-                f.related_model,
-                parent=self,
-                relation_field=name
+                f.related_model, parent=self, relation_field=name
             )
 
-            # Add model to models list so circular dependencies don't occur.
-            models.append(f.related_model)
-            node.expand(models=models)
+            # Prevent circular dependencies by adding the field to the list.
+            parsed_rel_fields.append(_get_f_key(self.model, f))
+            node.expand(parsed_rel_fields=parsed_rel_fields)
 
             # Append the completed child node (and tree) to the parent node.
             self.children.append(node)
 
+    def update_queryet(self, queryset, archived, point):
+        # If archiving the related objects. Then ensure the new `point`
+        # is added to the related objects.
+        if archived is True:
+            queryset.update(
+                archived=True,
+                archive_points=ArrayAppend('archive_points', point)
+            )
+
+        # If unarchiving the related objects. Then ensure the current
+        # `point` is removed from the related objects. Only set the
+        # archived field to false if there are no points.
+        elif archived is False:
+            queryset.update(
+                archived=Case(
+                    When(
+                        archive_points__contains=[point],
+                        archive_points__len=1,
+                        then=archived
+                    ),
+                    default=F('archived')
+                ),
+                archive_points=ArrayRemove('archive_points', point)
+            )
+
     def update(self, instance, archived=None):
         """
         Update the node by cascading down the tree. Requires an instance id.
@@ -142,29 +171,10 @@ class ArchiveNode():
             fields = self._get_relation_fields(node)
             filters["".join(("__".join(fields), '__id'))] = instance.id
 
-            # If archiving the related objects. Then ensure the new `point`
-            # is added to the related objects.
-            if archived is True:
-                node.model.objects.filter(**filters).update(
-                    archived=archived,
-                    archive_points=ArrayAppend('archive_points', point)
-                )
-
-            # If unarchiving the related objects. Then ensure the current
-            # `point` is removed from the related objects. Only set the
-            # archived field to false if there are no points.
-            if archived is False:
-                node.model.objects.filter(**filters).update(
-                    archived=Case(
-                        When(
-                            archive_points__contains=[point],
-                            archive_points__len=1,
-                            then=archived
-                        ),
-                        default=F('archived')
-                    ),
-                    archive_points=ArrayRemove('archive_points', point)
-                )
+            # Update the queryset.
+            self.update_queryset(
+                node.model.objects.filter(**filters), archived, point
+            )
 
             # Cascade down to next node.
             node.update(instance, archived)
@@ -228,30 +238,31 @@ class ArchiveModel(StateModel):
         transaction so that failed inserts are rolled back.
         """
 
-        # If the archived status has changed ensure all related objects are
-        # also altered accordingly.
-        if self.original and self.archived != self.original.archived:
-            node = ArchiveNode(self.__class__)
+        # Only fire off additional archive logic if this is an existing
+        # instance.
+        if not self._state.adding:
+            # Check if the archived value is changed from the original value.
+            if self.original and self.archived != self.original.archived:
+                node = ArchiveNode(self.__class__)
 
-            # Archive the object and all related/dependent objects.
-            if self.archived and not self.original.archived:
-                node.expand()
-                node.update(self, archived=True)
+                # Archive the object and all related/dependent objects.
+                if self.archived and not self.original.archived:
+                    node.expand()
+                    node.update(self, archived=True)
 
-            # Unarchive the object and all related/dependent objects.
-            elif not self.archived and self.original.archived:
-                # Ensure that a parent is not already archived.
-                if self.archive_points:
-                    raise CannotModifyObjectWithArchivedParentError()
+                # Unarchive the object and all related/dependent objects.
+                elif not self.archived and self.original.archived:
+                    # Ensure that a parent is not already archived.
+                    if self.archive_points:
+                        raise CannotModifyObjectWithArchivedParentError()
 
-                node.expand()
-                node.update(self, archived=False)
+                    node.expand()
+                    node.update(self, archived=False)
 
-        # Else if already archived then ensure modifications cannot be performed
-        # on the model instance (except for archive changes as above).
-        elif (not force
-                and (self._must_be_unarchived_to_modify and self.archived)):
-            raise CannotModifyArchivedObjectError()
+            # Check if the object may be modified due to the archived value.
+            elif (not force
+                    and (self._must_be_unarchived_to_modify and self.archived)):
+                raise CannotModifyArchivedObjectError()
 
         return super().save(*args, **kwargs)
 
